#!/usr/bin/python3
import statistics
import sys
import json
import itertools

MACHINES = [
        ('pi3', 'orange'),
        ('pi4', 'green'),
        ('grav2', 'red'),
        ('grav3', 'blue'),
        ]
VARIANTS = [
        'memcmp',
        'memcpy',
        'memcpy-random',
        'memmove',
        'memmove-large',
        'memmove-random',
    ]

STEPS = ['before', 'after']
FUNCTIONS = {
        #'min': (min, 'o'),
        #'mean': (statistics.mean, 's'),
        'median': (statistics.median, '+'),
        }

DISTROS = ['focal', 'jammy']

ITERATIONS = 100

label = "length"
# timings for one length are grouped in a list which is then add it to a dictionary that uses the length as key
def load(fname):
    r = {}
    global label
    label = "length"
    with open(fname) as fp:
        results = json.load(fp)['functions']
        if 'memmove' in results:
            if 'random' in results['memmove']['bench-variant']:
                label = "max src/dst offset"
            results = results['memmove']['results']
        elif 'memcpy' in results:
            if 'random' in results['memcpy']['bench-variant']:
                label = "max src/dst offset"
            results = results['memcpy']['results']
        elif 'memcmp' in results:
            results = results['memcmp']['results']
        else:
            Exception('could not find any of the known keys (memcpy, memcmp, memmove) in the json file')

    for result in results:
        time = result['timings']
        length = result['length']
        if length not in r:
            r[length]=[]
        r[length].append(float(time[0]))

    return r

#load all the runs into a new dict, as above: "128 : [<mean of the timings for length 128 of run 1>, <mean of the timings for length 128 of run 2>, .."
# for all lengths in the set of results ( roughly 0..4096, some lengths are skipped by the benchmarks)

def load_set_js(directory, iterations):
    data = {}

    for (machine, _), variant, step in itertools.product(MACHINES, VARIANTS, STEPS):
        fnames = [f"{directory}/results-{machine}/bench-{variant}-{step}.{i}.json" for i in range(1, iterations+1)]
        for fname in fnames:
            r=load(fname);
            mr = data.setdefault(variant, {}).setdefault(machine, {}).setdefault(step, {})
            for length in r:
                row = mr.setdefault(length, {})
                for key, (func, _) in FUNCTIONS.items():
                    row.setdefault(key, []).append(func(r[length]))

    return data

import matplotlib as mpl
import matplotlib.pyplot as plt
def exp2():
    if len(sys.argv) > 1:
        distros = [sys.argv[1]]
    else:
        distros = DISTROS
    if len(sys.argv) > 2:
        variants = [sys.argv[2]]
    else:
        variants = VARIANTS
    #iterations = int(sys.argv[2])
    for distro in distros:
        data = load_set_js(distro, ITERATIONS)

        #combinations = list(itertools.product(MACHINES, FUNCTIONS.keys()))
        for variant in variants:
            #print(f"{variant}" + ','.join(['' for _ in combinations]))
            #print("length," + ",".join([f"{machine}-{func}-{step}" for machine, func, step in combinations]))
            lengths = sorted(data[variant][MACHINES[0][0]][STEPS[0]].keys())
            fig, ax = plt.subplots()
            for (machine, color) in MACHINES:
                for func_name, (func, style) in FUNCTIONS.items():
                    d1 = data[variant][machine][STEPS[0]]
                    d2 = data[variant][machine][STEPS[1]]
                    values = [(func(d2[l][func_name]) - func(d1[l][func_name])) * 100.0 / func(d1[l][func_name]) for l in lengths]
                    ax.plot(lengths, values, style, color=color, label=f"{machine} {func_name}")
            ax.axhline(y=0, color='k')
            plt.title(f"{distro} {variant}")
            plt.legend(loc='best')
            plt.savefig(f"{distro}_{variant}.svg")
            plt.show()
            #for length in lengths:
            #    row = [length]
            #    for machine, func, step in combinations:
            #            row.append((row[-1] - row[-2]) * 100.0 / row[-2])
            #        else:
            #            row.append(FUNCTIONS[func](data[variant][machine][step][length][func]))
            #    #print(",".join([str(d) for d in row]))
exp2()

